;
;	                      PIC16F84.
;                              http://pic16f84.narod.ru
;                                 12.02.2002
;
;*********************************************************************************************************
;
;      .     
;    RS-232 .
;
;
;                               RS-232 .
;
;      :
;
;               _ClkIn          :          .
;                                       ( :    RC      
;                                            ).
;               _BaudRate       :         .     
;                                        .      
;                                         .   4  
;                                          600 .. 4800 .   600 .. 19200 
;                                        ,      10. 
;                                              .  
;                                       '_ClkIn'  '_BaudRate' ,    
;                                          .
;               _DataBits       :           1  8.
;               _StopBits       :         .     1.
;               _PARITY_ENABLE  :       '_PARITY_ENABLE' .     TRUE   FALSE. 
;                                            TRUE,   FALSE. 
;                                           '_ODD_PARITY'.
;               _ODD_PARITY     :        TRUE  FALSE.  TRUE,   
;                                         ,    .   
;                                         '_PARITY_ENABLE'   FALSE.
;               _USE_RTSCTS     :       TRS  CTS     .    
;                                         FALSE     .   
;                                       TRUE - TRS  CTS      PORTB. 
;
;
;                          /  ( Serial Status / Control Register )  :
;
;       "SerialStatus"          :   /  
;
;       Bit 0   :       _txmtProgress   (1 -   ,   . 0    
;                                       .)
;                                        ,        "PutChar",  
;                                          ,  ,   . 
;                                           ,     .
;       Bit 1   :       _txmtEnable           1   . 
;                                              .   
;                                             (  '_txmtProgress'  
;                                        1 )   '_txmtEnable'  0.    
;                                           'PutChar'.
;       Bit 2   :       _rcvProgress        1    .    0  
;                                             1     
;                                         .
;       Bit 3   :       _rcvOver        0   ,    .    
;                                             'GetChar'    
;                                        .    0, ,     'RxReg'.  
;                                            ,      
;                                         .
;       Bit 4   :       _ParityErr      1   ,        (  
;                                        ,       ( 'ODD' )    
;                                       ( 'EVEN' ) )      . 
;       Bit 5   :       _FrameErr       1       .
;
;       Bit 6   :       _unused_         .
;
;       Bit 7   :       _parityBit           .       
;                                            .     
;                                             .    
;                                         .
;
;
;           :
;                       1) ,   '_txmtProgress'  '_rcvOver' .
;                       2)     'TxReg'.
;                       3)   'PutChar'.
;
;           :
;                       1) ,   '_txmtProgress'  '_rcvOver' .
;                       2)   'GetChar'.
;                       3)       'RxReg',    '_revOver' .
;
;
;
;        :
;           oscillator  -  HS
;           power       -  on
;           wach        -  off
;           code protect - off
;
;*********************************************************************************************************

		Radix   DEC

	include   "p16c71.inc"    ;    


_ResetVector	set	0x00      ;   
_IntVector	set	0x04      ;     

TRUE	equ	1
FALSE	equ	0

DIGIT           EQU     1Dh       ;  
;SCRATCH         EQU     1Eh       ;  
;SCRATCH1        EQU     1Fh       ;  

;*********************************************************************************************************
;                               
;************************  RS-232  **************************************************

_ClkIn          equ     4000000         ;   4 
_BaudRate       set     1200            ;   (  ) 1200
_DataBits       set     8               ; 8  
_StopBits       set     1               ; 1  

#define _PARITY_ENABLE  FALSE           ;    
#define _ODD_PARITY     FALSE           ;      
#define _USE_RTSCTS     FALSE           ;    . 

;*********************************************************************************************************

	include   "rs232.h"       ;      RS-232

;*********************************************************************************************************


	ORG     _ResetVector            ;  
	goto    Start
;
	
	ORG     _IntVector              ;    
	goto    Interrupt
;
;*********************************************************************************************************


;**************************    *****************************************************

;        .    ,  
;    .     .   
;  ,      -  
;    .


Start:
	call    InitSerialPort       ;    

WaitForNextSel:
   if _USE_RTSCTS                ;    
	bcf     STATUS,RP0           ; ,   0
	bcf     _RTS                 ;   _RTS --     
   endif
	call    GetChar              ;   
	btfsc   _rcvOver             ;   _rcvOver ,    (    RxReg)
	goto    $-1                  ;       
;
;
	bcf     STATUS,RP0           ; ,   0
	movf    RxReg,W              ;      W
	movwf   DIGIT                ;      DIGIT
;	nop


	movf    DIGIT,W
	movwf   TxReg               ;      TxReg
   if _USE_RTSCTS                ;    
	bsf     _RTS                ;   _RTS --   ,     
	btfsc   _CTS                ;   CTS,      
	goto    $-1
   endif
	call    PutChar             ;    
	btfsc   _txmtProgress       ;   _txmtProgress ,    
	goto    $-1  
;
;************************    ************************************


	movf    DIGIT,W             ;     DIGIT   W
        SUBLW   D'101'              ;  D'101'  W
        BTFSS   STATUS,Z            ;     0   SVET1ON
        GOTO    a1                  ;    a1
        CALL    SVET1ON
        GOTO    aaa                 ;     
a1
	movf    DIGIT,W             ;     DIGIT   W
        SUBLW   D'102'              ;  D'102'  W  
        BTFSS   STATUS,Z            ;     0   SVET1OFF
        GOTO    a2
        CALL    SVET1OFF
        GOTO    aaa                 ;     
a2
	movf    DIGIT,W             ;     DIGIT   W
        SUBLW   D'201'              ;  D'201'  W   
        BTFSS   STATUS,Z            ;     0   SVET2ON
        GOTO    a3
        CALL    SVET2ON
        GOTO    aaa                 ;     
a3
	movf    DIGIT,W             ;     DIGIT   W
        SUBLW   D'202'              ;  D'202'  W   
        BTFSS   STATUS,Z            ;     0   SVET2OFF
        GOTO    a4
        CALL    SVET2OFF
        GOTO    aaa                 ;     
a4
	movf    DIGIT,W             ;     DIGIT   W
        SUBLW   D'001'              ;  D'001'  W   
        BTFSS   STATUS,Z            ;     0   SVET3ON
        GOTO    a5
        CALL    SVET3ON
        GOTO    aaa                 ;     
a5
	movf    DIGIT,W             ;     DIGIT   W
        SUBLW   D'002'              ;  D'002'  W   
        BTFSS   STATUS,Z            ;     0   SVET3OFF
        GOTO    aaa                 ;     
        CALL    SVET3OFF
        GOTO    aaa
aaa

;----------------------------------------------------------------------------------------------
;
	goto    WaitForNextSel      ;     
;
;********************     *******************************************



;********************    *************************************

SVET1OFF
	bcf     STATUS,RP0           ; ,   0 
	bcf     PORTB,3              ;  ,    RB3
RETURN
SVET1ON
	bcf     STATUS,RP0           ; ,   0 
	bsf     PORTB,3              ;  ,    RB3
RETURN
SVET2OFF
	bcf     STATUS,RP0           ; ,   0 
	bcf     PORTB,2              ;  ,    RB2 
RETURN
SVET2ON
	bcf     STATUS,RP0           ; ,   0 
	bsf     PORTB,2              ;  ,    RB2 
RETURN
SVET3OFF
	bcf     STATUS,RP0           ; ,   0 
	bcf     PORTA,3              ;  ,    RA3
RETURN
SVET3ON
	bcf     STATUS,RP0           ; ,   0 
	bsf     PORTA,3              ;  ,    RA3
RETURN

;********************     ******************************
           
;*********************************   **************************************
;
;     .          
;    .
;
;      :
;                          .    
;                        .      
;                              
;                      .
;     :
;                         ,      
;                         (   )   
;                       0xFF.         
;                        ,     . 
;                               . 
;                                .  
;                              
;                         .


Interrupt:
	btfss   INTCON,T0IF           ;     ? 
	retfie                        ;     ,  
;
;     W  STATUS
;
	movwf   SaveWReg              ;    W
	swapf   STATUS,W               
	movwf   SaveStatus            ;    STATUS 
;
	btfsc   _txmtProgress         ;    ?
	goto    _TxmtNextBit          ;   -   
	btfsc   _rcvProgress          ;    ?
	goto    _RcvNextBit           ;   -     ; Receive Next Bit
	goto    _SBitDetected         ;   - ,         
;
RestoreIntStatus:
	swapf   SaveStatus,W
	movwf   STATUS                ;   STATUS 
	swapf   SaveWReg, F           ;   W  
	swapf   SaveWReg,W            ;   W
	bcf     INTCON,T0IF           ;     
	retfie                        ;   ,  

;*********************     ********************************



;*********************    /  **********************************

;  TX (RB7)        .
;  RX (RA4)      .
;  RB2, RB3  RA   .

InitSerialPort:
	clrf    SerialStatus            ;   

	bcf     STATUS,RP0              ;  0
	bsf     TX                      ;   TX  1                       
	bsf     STATUS,RP0              ;  1     TrisA  TrisB
	bcf     TX                      ;   TX      TRIS
  if _USE_RTSCTS                    ;    
	bcf     _RTS                    ;   RTS ,  
	bsf     _CTS                    ;   CTS ,     
  endif
	bsf     RX_Pin                  ;   RX    

;---------------------------------------------------------------------
	bcf     PORTB,2                 ;   RB2  
	bcf     PORTB,3                 ;   RB3  
	bcf     PORTA,3                 ;   RA3  
	bcf     STATUS,RP0              ;  0
	bcf     PORTB,2                 ;  ,    RB2
	bcf     PORTB,3                 ;  ,    RB3
	bcf     PORTA,3                 ;  ,    RA3
;--------------------------------------------------------------------------

	return

;*********************      /  **************************

	include   "txmtr.asm"     ;      "txmtr.asm"
	include   "rcvr.asm"      ;     "rcvr.asm"

	END

; ********************   *******************************************************************


